@@ -12,6 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import datetime
 
 
 def run(args=None):
@@ -140,13 +141,10 @@ def add_task(args=None):
 
 
 def add_user(args=None):
-    import json
     import argparse
 
     from taskmanager import create_app
-    from taskmanager import models
-    from taskmanager import user_datastore
-    from flask_security import hash_password
+    from taskmanager.util.helpers import create_user
 
     parser = argparse.ArgumentParser(description="Add a user to the taskmanager.")
     parser.add_argument('-u', '--username', required=True, help="The username.")
@@ -155,30 +153,21 @@ def add_user(args=None):
     parser.add_argument('-e', '--email', required=True, help="The e-mail.")
     parser.add_argument('-w', '--assignment-weight', type=float, default=1.0, help="Assignment weighting", required=False)
     parser.add_argument('-i', '--inactive', default=False, action='store_true', help="If the user starts active or not", required=False)
+    parser.add_argument('-i', '--unconfirmed', default=False, action='store_true', help="If the user email is not confirmed", required=False)
     parser.add_argument('-f', '--force', action='store_true', default=False, help="Do not ask questions, just do it")
     args = parser.parse_args(args=args)
 
     app = create_app()
     with app.app_context():
-        db = models.db
-
-        user = {"username": args.username,
-                "password": hash_password(args.password),
-                "name": args.full_name,
-                "email": args.email,
-                "assignment_weight": args.assignment_weight,
-                "active": not args.inactive}
-        db_user = user_datastore.create_user(**user)
-
-        doit = False
-        if not args.force:
-            doit = input("Are you sure you want to commit user [{}], to database '{}' [yes/no]: ".format(user['username'], app.config['SQLALCHEMY_DATABASE_URI'])) == 'yes'
-        if doit or args.force:
-            db.session.commit()
-            print("\n * Committed to the database.")
-        else:
-            db.session.rollback()
-            print("\n * Cancelled.")
+        create_user(
+            username=args.username,
+            password=args.password,
+            name=args.full_name,
+            email=args.email,
+            confirm=not args.unconfirmed,
+            assignment_weight=args.assignment_weight,
+            active=not args.inactive,
+        )
 
 
 def config_from_file(args=None):
