@@ -13,12 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from datetime import datetime
+from string import Template
 import json
 
 from flask import Blueprint, abort, jsonify
-from flask_restplus import Api, Resource, fields, inputs, reqparse
+from flask_restx import Api, Resource, fields, inputs, reqparse
 from flask_security import auth_required, current_user, http_auth_required, permissions_accepted, permissions_required
 from flask_security.utils import hash_password
+from sqlalchemy import func
+from sqlalchemy.sql.expression import alias
 
 from . import RegisteredApi
 from .. import exceptions
@@ -130,10 +134,13 @@ task_post = api.model('TaskPost', {
     'project': fields.String,
     'template': fields.String,
     'content': fields.String,
+    'raw_content': fields.String,
     'tags': fields.List(ObjectUrl('api_v1.tag', attribute='id')),
     'callback_url': fields.String,
+    'raw_callback_url': fields.String,
     'callback_content': fields.String,
     'generator_url': fields.String,
+    'tracking_id': fields.String,
     'application_name': fields.String,
     'application_version': fields.String,
     'users': fields.List(ObjectUrl('api_v1.user', attribute='id')),
@@ -146,10 +153,13 @@ task_put = api.model('TaskPut', {
     'project': fields.String,
     'template': fields.String,
     'content': fields.String,
+    'raw_content': fields.String,
     'status': fields.String,
+    'raw_callback_url': fields.String,
     'callback_url': fields.String,
     'callback_content': fields.String,
     'generator_url': fields.String,
+    'tracking_id': fields.String,
     'application_name': fields.String,
     'application_version': fields.String,
     'tags': fields.List(ObjectUrl('api_v1.tag', attribute='id')),
@@ -174,9 +184,12 @@ class TaskListAPI(Resource):
     post_request_parser.add_argument('project', type=str, required=True, location='json')
     post_request_parser.add_argument('template', type=str, required=True, location='json')
     post_request_parser.add_argument('content', type=json_type, required=True, location='json')
+    post_request_parser.add_argument('raw_content', type=str, required=False, location='json')
     post_request_parser.add_argument('callback_url', type=str, required=False, location='json')
+    post_request_parser.add_argument('raw_callback_url', type=str, required=False, location='json')
     post_request_parser.add_argument('callback_content', type=str, required=False, location='json')
     post_request_parser.add_argument('generator_url', type=str, required=False, location='json')
+    post_request_parser.add_argument('tracking_id', type=str, required=False, location='json')
     post_request_parser.add_argument('application_name', type=str, required=False, location='json')
     post_request_parser.add_argument('application_version', type=str, required=False, location='json')
     post_request_parser.add_argument('tags', type=list, required=False, location='json')
@@ -226,10 +239,13 @@ class TaskListAPI(Resource):
         try:
             task = control.insert_task(
                 content=args['content'],
+                raw_content = args['raw_content'],
                 project=args['project'],
                 callback_url=args['callback_url'],
+                raw_callback_url=args['raw_callback_url'],
                 callback_content=args['callback_content'],
                 template=args['template'],
+                tracking_id=args['tracking_id'],
                 generator_url=args['generator_url'],
                 tags=args['tags'],
                 users=args['users'],
@@ -258,8 +274,10 @@ class TaskAPI(Resource):
     request_parser.add_argument('project', type=str, required=False, location='json')
     request_parser.add_argument('template', type=str, required=False, location='json')
     request_parser.add_argument('content', type=json_type, required=False, location='json')
+    request_parser.add_argument('raw_content', type=str, required=False, location='json')
     request_parser.add_argument('status', type=str, required=False, location='json')
     request_parser.add_argument('callback_url', type=str, required=False, location='json')
+    request_parser.add_argument('raw_callback_url', type=str, required=False, location='json')
     request_parser.add_argument('callback_content', type=str, required=False, location='json')
     request_parser.add_argument('generator_url', type=str, required=False, location='json')
     request_parser.add_argument('application_name', type=str, required=False, location='json')
@@ -338,7 +356,14 @@ class TaskAPI(Resource):
                 if not isinstance(content, str):
                     content = json.dumps(content)
 
-                task.content = content
+                if args['raw_content'] is None:
+                    task.raw_content = content
+            else:
+                abort(403, "You are not authorized to update the content")
+        
+        if args['raw_content'] is not None:
+            if has_permission_any('task_update_all'):
+                task.raw_content = args['raw_content']
             else:
                 abort(403, "You are not authorized to update the content")
 
@@ -351,10 +376,16 @@ class TaskAPI(Resource):
 
         if args['callback_url'] is not None:
             if has_permission_any('task_update_all'):
-                task.callback_url = args['callback_url']
+                if args['raw_callback_url'] is None:
+                    task.raw_callback_url = args['callback_url']
             else:
                 abort(403, "You are not authorized to update the callback url")
 
+        if args['raw_callback_url'] is not None:
+            if has_permission_any('task_update_all'):
+                task.raw_callback_url = args['raw_callback_url']
+            else:
+                abort(403, "You are not authorized to update the callback url")
 
         if args['callback_content'] is not None:
             if has_permission_any('task_update_all'):
@@ -1420,3 +1451,143 @@ class GroupUsersAPI(Resource):
                 abort(403)
 
         return group
+
+meta_history_fields = api.model('MetaHistory',{
+    'id': fields.Integer,
+    'timestamp': fields.DateTime,
+    'value': fields.String,
+})
+
+meta_fields = api.model('Meta', {
+    'id': fields.Integer,
+    'label': fields.String,
+    'value': fields.String,
+    'history': fields.List(fields.Nested(meta_history_fields))
+})
+
+meta_list = api.model('MetaList', {
+    'meta': fields.List(fields.Nested(meta_fields)),
+})
+
+meta_put = api.model('MetaPut', {
+    'label': fields.String,
+    'value': fields.String
+})
+
+@api.route('/meta', endpoint='meta')
+class MetaListAPI(Resource):
+    post_request_parser = reqparse.RequestParser()
+    post_request_parser.add_argument('label', type=str, required=True, location='json')
+    post_request_parser.add_argument('value', type=str, required=True, location='json')
+    
+    @http_auth_required
+    @permissions_accepted('meta_read_all')
+    @api.marshal_with(meta_list)
+    @api.response(200, "Success")
+    @api.response(403, "You are not authorized to get this information")
+    def get(self):
+        meta = models.Meta.query.all()
+        
+        if not has_permission_any('meta_read_all'):
+            abort(403)
+
+        return {"meta": meta}
+
+    @http_auth_required
+    @permissions_accepted('meta_add')
+    @api.marshal_with(meta_fields)
+    @api.expect(meta_put)
+    @api.response(201, "Added new Meta entry")
+    @api.response(403, "You are not authorized to do this operation")
+    def post(self):  
+        args = self.post_request_parser.parse_args()
+
+        meta = models.Meta(label=args['label'], value=args['value'])
+        
+        db.session.add(meta)
+        db.session.commit()
+        db.session.refresh(meta)
+
+        return meta, 201
+
+@api.route('/meta/<int:id>')
+class MetaAPI(Resource):
+    request_parser = reqparse.RequestParser()
+    request_parser.add_argument('value', type=str, required=True, location='json')
+    
+    @http_auth_required
+    @permissions_accepted('meta_read_all')
+    @api.marshal_with(meta_fields)
+    @api.response(200, "Success")
+    @api.response(403, "You are not authorized to get this information")
+    def get(self, id):
+        meta = models.Meta.query.filter(models.Meta.id == id).one_or_none()
+        if meta is None:
+            abort(404)
+
+        if not has_permission_any('meta_read_all'):
+            abort(403)
+
+        return meta
+
+    @http_auth_required
+    @permissions_accepted('meta_update')
+    @api.marshal_with(meta_fields)
+    @api.expect(meta_put)
+    @api.response(200, "Success")
+    @api.response(404, "Could not find Meta entry")
+    def put(self, id):
+        meta = models.Meta.query.filter(models.Meta.id == id).one_or_none()
+        if meta is None:
+            abort(404)
+
+        args = self.request_parser.parse_args()
+
+        meta.value = args['value']
+        
+        db.session.commit()
+        db.session.refresh(meta)
+
+        return meta
+
+
+@api.route('/task-at-timepoint/<int:id>', endpoint='task-at-timepoint')
+class TaskAtTimepointAPI(Resource):
+    request_parser = reqparse.RequestParser()
+    request_parser.add_argument('timestamp', type=inputs.datetime_from_iso8601, required=False, default=datetime.now(), location='args', help="Date and time on which you'd like to calculate the task contents (ISO8601)")
+    
+    @http_auth_required
+    @permissions_accepted('task_read', 'task_read_all')
+    @api.marshal_with(task_get)
+    @api.response(200, 'Success')
+    @api.response(404, 'Could not find selected task')
+    def get(self, id):
+        args = self.request_parser.parse_args()
+        filter_timestamp = args['timestamp']
+        
+        query = models.Task.query.filter(models.Task.id == id)
+
+        if not current_user.has_permission('task_read_all'):
+            tasks_via_user_query = query.filter(models.Task.users.contains(current_user))
+            tasks_via_group_query = query.filter(models.Task.users_via_group.contains(current_user))
+            query = tasks_via_user_query.union(tasks_via_group_query)
+        task = query.one_or_none()
+
+        if task is None:
+            abort(404)
+        
+        # Find meta history entries with latest values before requested timestamp
+        latest_timestamps = db.session.query(models.MetaHistory.meta_id, func.max(models.MetaHistory.timestamp).label("maxtimestamp")).filter(models.MetaHistory.timestamp<filter_timestamp).group_by(models.MetaHistory.meta_id).subquery()
+
+        # Find values for corresponding entries and join with labels.
+        latest_meta = db.session.query(models.Meta.label, latest_timestamps.c.meta_id, models.MetaHistory.value).join(models.MetaHistory, (latest_timestamps.c.meta_id==models.MetaHistory.meta_id) & (latest_timestamps.c.maxtimestamp==models.MetaHistory.timestamp)).join(models.Meta, latest_timestamps.c.meta_id==models.Meta.id).all()
+
+        mapping = {obj.label: obj.value for obj in latest_meta}
+
+        content_template = Template(task.raw_content)
+        callback_template = Template(task.raw_callback_url)
+        
+        task.content = None if task.raw_content is None else content_template.safe_substitute(**mapping)
+        task.callback_url = None if task.raw_callback_url is None else callback_template.safe_substitute(**mapping)
+
+        return task
\ No newline at end of file
