@@ -13,13 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask_security.forms import LoginForm
+from flask_security.forms import LoginForm, RegisterForm, unique_identity_attribute
 
-from wtforms import StringField, PasswordField, BooleanField, SubmitField
+from wtforms import StringField
 from wtforms.validators import DataRequired
 
+
 class AuthLoginForm(LoginForm):
-    username = StringField('Username', validators=[DataRequired()])
-    password = PasswordField('Password', validators=[DataRequired()])
-    remember_me = BooleanField('Remember Me')
-    submit = SubmitField('Login')
+    email = StringField('Email/Username', validators=[DataRequired()])
+
+
+class AuthRegisterForm(RegisterForm):
+    username = StringField('Username', validators=[DataRequired(), unique_identity_attribute])
+    name = StringField('Name', validators=[DataRequired()])
