@@ -86,6 +86,9 @@ def insert_task(content: Union[str, dict],
                 callback_content: str,
                 template: str,
                 generator_url: str,
+                tracking_id: str=None,
+                raw_content: str=None,
+                raw_callback_url: str=None,
                 tags: Sequence[str]=None,
                 users: Sequence[str]=None,
                 groups: Sequence[str]=None,
@@ -135,11 +138,17 @@ def insert_task(content: Union[str, dict],
     if groups is not None:
         groups = [get_object_from_arg(g, models.Group, models.Group.groupname) for g in groups]
 
+    raw_content = raw_content if raw_content is not None else content
+    raw_callback_url = raw_callback_url if raw_callback_url is not None else callback_url
+
     # Create task object
     task = models.Task(project=project,
                        template=template,
                        content=content,
+                       raw_content=raw_content,
+                       tracking_id = tracking_id,
                        callback_url=callback_url,
+                       raw_callback_url=raw_callback_url,
                        callback_content=callback_content,
                        generator_url=generator_url,
                        application_name=application_name,
