@@ -16,12 +16,16 @@
 import datetime
 import threading
 
+from string import Template
+
 from flask import current_app
 from flask_sqlalchemy import SQLAlchemy
 from flask_security import SQLAlchemyUserDatastore
 
 from sqlalchemy.sql import func
 from sqlalchemy.orm.exc import NoResultFound
+from sqlalchemy import event
+from sqlalchemy.orm.attributes import get_history
 
 from .callbacks import dispatch_callback
 from .auth.models import BaseUser
@@ -38,8 +42,9 @@ class Role(db.Model, BaseRole):
     """ This implements the BaseRole from the .auth.models module.
     In this specific case, the BaseRole is sufficient. """
     __tablename__ = 'role'
-    pass
 
+    def __repr__(self):
+        return f'<{self.name}>'
 
 class TaskGroupLinks(db.Model):
     __tablename__ = 'task_group_links'
@@ -73,6 +78,9 @@ class TaskTemplate(db.Model):
         self.label = label
         self.content = content
 
+    def __repr__(self):
+        return f'<TaskTemplate {self.label}>'
+
 
 class Task(db.Model):
     id = db.Column(db.Integer, primary_key=True)
@@ -81,43 +89,26 @@ class Task(db.Model):
     lock_id = db.Column(db.Integer, db.ForeignKey('user.id', name="fk_task_lock_id_user"))
     lock = db.relationship("User", uselist=False)
     content = db.Column(db.Text)
+    raw_content = db.Column(db.Text)
     callback_url = db.Column(db.Text)
+    raw_callback_url = db.Column(db.Text)
     callback_content = db.Column(db.Text)
     tags = db.relationship("Tag", secondary="tag_task_links", backref=db.backref("tasks"))
     parent_id = db.Column(db.Integer, db.ForeignKey('task_group.id', name='fk_task_parent_id_task_group'))
     parent = db.relationship("TaskGroup", back_populates="tasks")
     template_id = db.Column(db.Integer, db.ForeignKey('task_template.id', name='fk_task_template_id_task_template'))
     template = db.relationship("TaskTemplate", back_populates="tasks")
+    tracking_id = db.Column(db.VARCHAR(100), nullable=False, default='unassigned')
     create_time = db.Column(db.DateTime(timezone=True), default=func.now())
     update_time = db.Column(db.DateTime(timezone=True), default=func.now(), onupdate=func.current_timestamp())
     generator_url = db.Column(db.String(length=512), nullable=True)
     application_name = db.Column(db.VARCHAR(length=32), nullable=True)
     application_version = db.Column(db.VARCHAR(length=16), nullable=True)
 
-    def __init__(self,
-                 project,
-                 template,
-                 content,
-                 callback_url=None,
-                 callback_content=None,
-                 generator_url=None,
-                 application_name=None,
-                 application_version=None):
-        self.project = project
-        self.template = template
-        self.content = content
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         self.status = 'queued'
-
-        if callback_url is not None:
-            self.callback_url = callback_url
-
-        if callback_content is not None:
-            self.callback_content = callback_content
-
-        self.generator_url = generator_url
-        self.application_name = application_name
-        self.application_version = application_version
-
+    
     def __repr__(self):
         return f'<Task {self.id}, project: {self.project}, template: {self.template.label}>'
 
@@ -178,9 +169,17 @@ class TaskGroup(db.Model):
 
     @property
     def status(self):
-        return 'done' if all(t.status == 'done' for t in self.tasks) else 'queued'
+        if all(t.status == 'done' for t in self.tasks):
+            return 'done' 
+        elif any(t.status == 'aborted' for t in self.tasks):
+            return 'aborted'
+        else:
+            return 'queued'
 
     def child_updated(self):
+        if any(x.status == 'aborted' for x in self.tasks):
+            for x in self.tasks:
+                if x.status != 'aborted': x.status = 'aborted'
         if all(x.status == 'done' for x in self.tasks):
             if self.callback_url is not None and not self.callback_fired:
                 self.callback_fired = True
@@ -205,13 +204,6 @@ class User(db.Model, BaseUser):
     roles = db.relationship('Role', secondary='roles_users',
                             backref=db.backref('users', lazy='dynamic'))
 
-    #def __init__(self, username, password, name, email, active=True, roles="user", assignment_weight=None):
-    #    current_app.logger.info(f"{roles}")
-    #    #BaseUser.__init__(self, username, password, name, email, roles)
-    #    
-    #    if self.assignment_weight is None:
-    #        self.assignment_weight = assignment_weight
-
     def __repr__(self):
         return f'<User {self.username} ({self.name})>'
 
@@ -253,9 +245,11 @@ class Group(db.Model):
     tasks = db.relationship("Task", secondary="task_group_links", backref=db.backref("groups"))
     create_time = db.Column(db.DateTime(timezone=True), default=func.now())
 
-    def __init__(self, groupname, name=None, members=None):
-        self.groupname = groupname
-        self.name = name
+    def __repr__(self):
+        return f'<Group {self.groupname} ({self.name})>'
+
+    def __init__(self, members=None, **kwargs):
+        super().__init__(**kwargs)
 
         if members is not None:
             for member in members:
@@ -263,8 +257,105 @@ class Group(db.Model):
                     user = User.query.filter(User.username == member).one()
                 except NoResultFound:
                     raise NoResultFound(f"User [{member}] is not found in the taskmanager")
-
                 self.users.append(user)
 
+
+class Meta(db.Model):
+    id = db.Column(db.INTEGER, primary_key=True)
+    label = db.Column(db.VARCHAR(128), unique=True, nullable=False)
+    value = db.Column(db.Text)
+    history = db.relationship('MetaHistory', backref='meta', lazy=True)
+
     def __repr__(self):
-        return f'<Group {self.groupname} ({self.name})>'
+        return f"<Meta {self.label}: {self.value}>"
+
+
+class MetaHistory(db.Model):
+    id = db.Column(db.INTEGER, primary_key=True)
+    meta_id = db.Column(db.INTEGER, db.ForeignKey('meta.id'), nullable=False)
+    timestamp = db.Column(db.DateTime(timezone=True), default=func.now())
+    value = db.Column(db.Text)
+
+    def __repr__(self):
+        return f"<Meta history - {self.timestamp}: {self.value}>"
+    
+
+@event.listens_for(Meta, "after_insert")
+@event.listens_for(Meta, "after_update")
+@event.listens_for(Meta, "after_delete")
+def after_meta_change(mapper, connection, target):
+    tasks = Task.query.all()
+    task_table = Task.__table__
+    meta = {obj.label: obj.value for obj in Meta.query.all()}
+    
+    for task in tasks:
+        content_template = Template(task.raw_content)
+        callback_template = Template(task.raw_callback_url)
+        
+        gen_content = None if task.raw_content is None else content_template.safe_substitute(**meta)
+        gen_callback_url = None if task.raw_callback_url is None else callback_template.safe_substitute(**meta)
+
+        connection.execute(
+            task_table.update().
+                where(task_table.c.id==task.id).
+                values(content=gen_content, callback_url=gen_callback_url)
+        )
+    
+@event.listens_for(Meta, "after_insert")
+@event.listens_for(Meta, "after_update")
+def log_history(mapper, connection, target):
+    meta_history_table = MetaHistory.__table__
+    value_changed = get_history(target, 'value').unchanged == ()
+    if value_changed:
+        connection.execute(
+            meta_history_table.insert().values(meta_id=target.id, value=target.value)
+        )
+
+@event.listens_for(Task, "before_update")
+def before_task_update(mapper, connection, target):
+    meta = {obj.label: obj.value for obj in Meta.query.all()}
+    update_content(target, meta)
+    update_callback_url(target, meta)
+
+
+def update_content(task, meta):
+    raw_content_changed = get_history(task, 'raw_content').unchanged == ()
+    content_changed = get_history(task, 'content').unchanged == ()
+
+    if raw_content_changed and content_changed:
+        print("Warning: You have tried to change both content and raw_content field. raw_content field is set to content.")
+
+    if content_changed:
+        task.raw_content = task.content
+    elif raw_content_changed:
+        content_template = Template(task.raw_content)
+        task.content = None if task.raw_content is None else content_template.safe_substitute(**meta)
+
+
+def update_callback_url(task, meta):
+    raw_callback_url_changed = get_history(task, 'raw_callback_url').unchanged == ()
+    callback_url_changed = get_history(task, 'callback_url').unchanged == ()
+    
+    if raw_callback_url_changed and callback_url_changed:
+        print("Warning: You have tried to change both callback_url and raw_callback_url field. raw_callback_url is set to callback_url.")
+
+    if callback_url_changed:
+        task.raw_callback_url = task.callback_url
+    elif raw_callback_url_changed:
+        callback_template = Template(task.raw_callback_url)
+        task.callback_url = None if task.raw_callback_url is None else callback_template.safe_substitute(**meta)
+
+
+@event.listens_for(Task, "before_insert")
+def before_task_insert(mapper, connection, target):
+    meta = {obj.label: obj.value for obj in Meta.query.all()}
+    
+    if target.raw_content is None:
+        target.raw_content = target.content
+    elif target.content is None:
+        target.content = Template(target.raw_content).safe_substitute(**meta)
+
+    if target.raw_callback_url is None:
+        target.raw_callback_url = target.callback_url    
+    elif target.callback_url is None:
+        target.callback_url = Template(target.raw_callback_url).safe_substitute(**meta)
\ No newline at end of file
