@@ -12,12 +12,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import datetime
 import os
 import yaml
-import string
 import pprint
-import random
+from typing import Optional
 
 from flask_security import SQLAlchemyUserDatastore
 from flask_security.utils import hash_password
@@ -70,6 +69,44 @@ def get_object_from_arg(id, model, model_name=None, skip_id=False, allow_none=Fa
     return data
 
 
+def create_user(username: str,
+                name: str,
+                password: str,
+                email: str,
+                assignment_weight: Optional[float] = 1.0,
+                active: bool = True,
+                confirm: bool = True,
+                force: bool = False):
+    """Helper to create a user"""
+    password = hash_password(password)
+    confirmed_at = datetime.datetime.now() if confirm else None
+
+    user_datastore = SQLAlchemyUserDatastore(models.db, models.User, models.Role)
+
+    db_user = user_datastore.create_user(
+        username=username,
+        name=name,
+        password=password,
+        email=email,
+        active=active,
+        confirmed_at=confirmed_at,
+        assignment_weight=assignment_weight,
+    )
+
+    doit = False
+    if not force:
+        doit = input(f"Are you sure you want to commit user [{db_user['username']}], to database [yes/no]: ") == 'yes'
+
+    if doit or force:
+        models.db.session.commit()
+        print("\n * Committed to the database.")
+    else:
+        models.db.session.rollback()
+        print("\n * Cancelled.")
+
+    return db_user
+
+
 def load_config_file(app, file_path, silent=False):
     file_path = str(file_path)
 
@@ -77,6 +114,7 @@ def load_config_file(app, file_path, silent=False):
         db = models.db
 
         user_datastore = SQLAlchemyUserDatastore(db, models.User, models.Role)
+
         if not os.path.isfile(file_path):
             db.session.rollback()
             raise ValueError(f"The file ({file_path}) does not exist")
@@ -90,37 +128,51 @@ def load_config_file(app, file_path, silent=False):
                 print("\n Adding Roles:")
             roles = {}
             for role in config["roles"]:
-                roles[role['name']] = user_datastore.create_role(**role)
-                if not silent:
-                    print(f"{roles[role['name']]} {role}")
+                if models.Role.query.filter(models.Role.name == role['name']).count() == 0:
+                    roles[role['name']] = user_datastore.create_role(**role)
+                    if not silent:
+                        print(f"{roles[role['name']]} {role}")
+                else:
+                    print(f"[WARNING] Skipping Role {role['name']}, already exists!")
             db.session.commit()
 
         if 'users' in config:
             if not silent:
                 print("\nAdding users:")
             for user in config['users']:
-                user['password'] = hash_password(user['password'])
-                db_user = user_datastore.create_user(**user)
-                if not silent:
-                    print(f"Adding {db_user}")
+                # Make sure user is confirmed
+                user['confirmed_at'] = datetime.datetime.now()
+                if models.User.query.filter((models.User.username == user['username']) | (models.User.email == user['email'])).count() == 0:
+                    user['password'] = hash_password(user['password'])
+                    db_user = user_datastore.create_user(**user)
+                    if not silent:
+                        print(f"Adding {db_user}")
+                else:
+                    print(f"[WARNING] Skipping User {user['username']} / {user['email']}, user and/or username already used!")
             db.session.commit()
 
         if 'groups' in config:
             if not silent:
                 print("\nAdding groups:")
             for group in config['groups']:
-                db.session.add(models.Group(**group))
-                if not silent:
-                    print(f"* {group}")
+                if models.Group.query.filter((models.Group.groupname == group['groupname']) | (models.Group.name == group['name'])).count() == 0:
+                    db.session.add(models.Group(**group))
+                    if not silent:
+                        print(f"* {group}")
+                else:
+                    print(f"[WARNING] Skipping Group {group['groupname']} already exists!")
             db.session.commit()
 
         if 'tags' in config:
             if not silent:
                 print("\nAdding Tags:")
             for tag in config['tags']:
-                db.session.add(models.Tag(tag))
-                if not silent:
-                    print(f"* {tag}")
+                if models.Tag.query.filter(models.Tag.name == tag).count() == 0:
+                    db.session.add(models.Tag(tag))
+                    if not silent:
+                        print(f"* {tag}")
+                else:
+                    print(f"[WARNING] Skipping Tag {tag}, already exists!")
             db.session.commit()
         
         if 'templates' in config:
@@ -135,9 +187,13 @@ def load_config_file(app, file_path, silent=False):
 
                 with open(template_path) as fh:
                     template_content = yaml.safe_load(fh)
-                    db.session.add(models.TaskTemplate(template_content['template_name'], yaml.safe_dump(template_content)))
-                if not silent:
-                    print(f"* Adding [{template_content['template_name']}] from {template_path}")
+                    template_name = template_content['template_name']
+                    if models.TaskTemplate.query.filter(models.TaskTemplate.label == template_name).count() == 0:
+                        db.session.add(models.TaskTemplate(template_name, yaml.safe_dump(template_content)))
+                        if not silent:
+                            print(f"* Adding [{template_name}] from {template_path}")
+                    else:
+                        print(f"[WARNING] Skipping Template {template_name}, already exists!")
 
         if not silent:
             print("\nCommitting to the database ...")
