@@ -21,14 +21,18 @@ from flask import redirect
 from flask import render_template
 from flask import request
 from flask import url_for
-from flask_restplus import reqparse
+from flask_restx import reqparse
 from flask_security import current_user
 from flask_security import login_required
 from flask_security import permissions_required
 from flask_security import permissions_accepted
+from flask_wtf import FlaskForm
+from wtforms import TextField, TextAreaField, SubmitField, SelectMultipleField
+from wtforms.validators import DataRequired
 
 from . import control, models
-
+from .util.mail import send_bulk_mail
+from .util.helpers import get_object_from_arg
 
 # Create the web blueprint
 bp = Blueprint('web', __name__)
@@ -251,4 +255,59 @@ def tag(id):
     if data is None:
         abort(404)
 
-    return render_template('tag.html', data=data)
\ No newline at end of file
+    return render_template('tag.html', data=data)
+
+
+class MailForm(FlaskForm):
+    recipients = SelectMultipleField('Recipients')
+    subject = TextField('Subject', validators=[DataRequired()])
+    body = TextAreaField('Message', validators=[DataRequired()])
+    submit = SubmitField('Send')
+
+    @bp.route("/mail", methods=['GET', 'POST'])
+    @login_required
+    @permissions_accepted('user_read_all')
+    def mail():
+        form = MailForm(request.form)
+
+        choices = [('A_all', 'All users')]
+
+        for group in models.Group.query.order_by(models.Group.name).all():
+            choices.append((f'G_{group.groupname}', f'Group: {group.name} <{group.groupname}>'))
+
+        for user in models.User.query.order_by(models.User.name).all():
+            choices.append((f'U_{user.username}', f'{user.name} <{user.email}>'))
+
+        form.recipients.choices = choices
+
+        if form.validate_on_submit():
+            recipients = form.recipients.data
+            subject = form.subject.data
+            body = form.body.data
+
+            new_recipients = set()
+            for recipient in recipients:
+                if recipient.startswith('A_'):
+                    for user in models.User.query.all():
+                        new_recipients.add(user)
+                elif recipient.startswith('G_'):
+                    group = get_object_from_arg(recipient[2:], models.Group, models.Group.groupname, skip_id=True)
+                    for user in group.users:
+                        new_recipients.add(user)
+                elif recipient.startswith('U_'):
+                    user = get_object_from_arg(recipient[2:], models.User, models.User.username, skip_id=True)
+                    new_recipients.add(user)
+
+            recipients = sorted(new_recipients, key=lambda u: u.username)
+
+            send_bulk_mail(
+                recipients=recipients,
+                subject=subject,
+                body=body,
+            )
+            return render_template('mail_sent.html',
+                                   recipients=recipients,
+                                   subject=subject,
+                                   body=body)
+
+        return render_template('mail.html', mail_form=form)
\ No newline at end of file
